@@ -156,16 +156,16 @@ sub prepare {
     ## Setup BLOCKS
     ##
     my $bp = $self->{BLOCKS} = [ do {
-	if ($self->{all} or $self->{only}) {	# --all, --only-matching
-	    ( [ 0, length ] );
-	}
-	elsif (@{$self->{block}}) {		# --block
+	if (@{$self->{block}}) {		# --block
 	    my $text = \$_;
 	    merge_regions { nojoin => 1, destructive => 1 }, map {
 		grep { $_->[0] != $_->[1] }
 		get_regions($self->{filename}, $text, $_);
 	    } @{$self->{block}};
 	}
+	elsif ($self->{all} or $self->{only}) {	# --all, --only-matching
+	    ( [ 0, length ] );
+	}
 	elsif (@blocks) {			# from matched range
 	    my %opt = ( A => $self->{after},
 		        B => $self->{before},
@@ -262,6 +262,12 @@ sub compose {
 	    push @list, [ $bp->[$bi], @matched ];
 	}
     }
+    ##
+    ## in the case --all and --block both specified
+    ##
+    if ($self->{all} and @list > 1) {
+	@list = ( [ [ 0, length ], map { shift @$_; @$_ } @list ] );
+    }
 
     ##
     ## ( [ [blockstart, blockend], [start, end], [start, end], ... ],
